# -*- coding: utf-8 -*-
import sys
import shlex
from subprocess import CalledProcessError
from subprocess import Popen
from subprocess import PIPE
import logging

from sudo import Sudo

log = logging.getLogger("pake.shell")


def debug_output(out):
    """
        | If output more than 30 lines
        | Split command debug output
        | Put first 15 lines and last 15 lines
    """
    split = out.split("\n")
    if len(split) > 30:
        out = split[:15] + [u"..."] + split[-15:]
        out = "\n".join(out)
    if out:
        log.debug("Out:\n{}".format(out))


class ShellPipe(object):
    def __init__(self, cmd):
        self._prev_cmds = []
        self._cmd = Popen(cmd, stdout=PIPE, stderr=PIPE)
        self._report = [" ".join(cmd)]

    def pipe(self, cmd):
        """
            Add command to pipe
        """
        prev = self._cmd
        self._prev_cmds.append(prev)
        self._cmd = Popen(cmd, stdin=prev.stdout, stdout=PIPE, stderr=PIPE)
        prev.stdout.close()
        self._report.append(" ".join(cmd))
        return self

    def last(self, cmd):
        """
            | log.info full pipe
            | Return pipe result
        """
        self.pipe(cmd)
        log.info("$ " + " | ".join(self._report))
        out, err = self._cmd.communicate()
        [c.wait() for c in self._prev_cmds]
        # если код не 0, то выполнилось неудачно
        if self._cmd.returncode or len([c.returncode for c in self._prev_cmds if c.returncode]):
            raise CalledProcessError(1024, err)
        # если есть, то это warning message
        if err.strip():
            print err
        debug_output(out)
        return out

    def __getattr__(self, attr):
        return ShellCmd(attr, pipe=self)


class ShellCmd(object):
    """
        Constructor for shell commands
    """
    def __init__(self, name, pipe=None):
        self._name = name
        self._pipe = pipe

    def _call(self, cmd, pipe):
        if pipe:
            return ShellPipe(cmd)
        else:
            log.info("$ " + " ".join(cmd))
            p = Popen(cmd, stdout=PIPE, stderr=PIPE)
            out, err = p.communicate()
            p.wait()
            # если код не 0, то выполнилось неудачно
            if p.returncode:
                raise CalledProcessError(p.returncode, err)
            # если есть, то это warning message
            if err.strip():
                print err
            debug_output(out)
            return out

    def _call_in_pipe(self, cmd, pipe):
        if pipe:
            return self._pipe.pipe(cmd)
        else:
            return self._pipe.last(cmd)

    def __call__(self, args="", pipe=False):
        cmd = u"{} {}".format(self._name, args)
        cmd = shlex.split(cmd)
        if not self._pipe:
            cmd = Sudo() + cmd
        if self._pipe:
            return self._call_in_pipe(cmd, pipe)
        else:
            return self._call(cmd, pipe)
